#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os
import shutil

#------------------------------------------------------------------------------#

# Load CSM file
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = os.path.join("..","ESP","transport.csm"),
                             outLevel = 0)

# Create an alias for geometry
wing = capsProblem.geometry

# Enable structures and CFD views
wing.cfgpmtr.VIEW.Concept     = 0
wing.cfgpmtr.VIEW.Cantilever  = 1
wing.cfgpmtr.VIEW.CfdInviscid = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

# reduce number of ribs for mASTROS
wing.cfgpmtr.wing.nrib1       = 1
wing.cfgpmtr.wing.nrib2       = 2
wing.cfgpmtr.wing.nrib3       = 4

wing.view()

#------------------------------------------------------------------------------#

# Set the number of transfer iterations
numTransferIteration = 2

#------------------------------------------------------------------------------#

# Create aflr4 AIM
aflr4 = capsProblem.analysis.create(aim = "aflr4AIM",
                                    name = "aflr4")

# Farfield growth factor
aflr4.input.ff_cdfr = 1.4

# Scaling factor to compute AFLR4 'ref_len' parameter
aflr4.input.Mesh_Length_Factor = 5

# Edge mesh spacing discontinuity scaled interpolant and farfield BC
aflr4.input.Mesh_Sizing = {"leftWing": {"edgeWeight":1.0},
                           "riteWing": {"edgeWeight":1.0},
                           "Farfield": {"bcType":"Farfield"}}

#------------------------------------------------------------------------------#

# Create AFLR3 AIM
aflr3 = capsProblem.analysis.create(aim  = "aflr3AIM",
                                    name = "aflr3")

# Link the surface mesh
aflr3.input["Surface_Mesh"].link(aflr4.output["Surface_Mesh"])

#------------------------------------------------------------------------------#

# Create fun3d AIM
fun3d = capsProblem.analysis.create(aim  = "fun3dAIM",
                                    name = "fun3d")

# Link the volume mesh
fun3d.input["Mesh"].link(aflr3.output["Volume_Mesh"])

# Set inputs for fun3d
speedofSound = 340.0 # m/s
refVelocity = 100.0 # m/s
refDensity = 1.2 # kg/m^3

fun3d.input.Proj_Name       = "aeroelasticModal"

fun3d.input.Mach            = refVelocity/speedofSound
fun3d.input.Equation_Type   = "compressible"
fun3d.input.Viscous         = "inviscid"
fun3d.input.Num_Iter        = 10
fun3d.input.Time_Accuracy   = "2ndorderOPT"
fun3d.input.Time_Step       = 0.001*speedofSound
fun3d.input.Num_Subiter     = 30
fun3d.input.Temporal_Error  = 0.01

fun3d.input.CFL_Schedule      = [1, 5.0]
fun3d.input.CFL_Schedule_Iter = [1, 40]
fun3d.input.Overwrite_NML     = True
fun3d.input.Restart_Read      = "off"

inviscidBC = {"bcType" : "Inviscid"}
fun3d.input.Boundary_Condition = {"Wing": inviscidBC,
                                  "Farfield": "farfield"}

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM")

# No Tess vertexes on edges (minimial mesh)
tess.input.Edge_Point_Max = 2

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create ASTROS AIMs
astros = capsProblem.analysis.create(aim  = "astrosAIM",
                                     name = "astros", autoExec = False)

# Link the surface mesh
astros.input["Mesh"].link(tess.output["Surface_Mesh"])

# Set inputs for astros
astros.input.Proj_Name = fun3d.input.Proj_Name

# Set analysis type
astros.input.Analysis_Type = "Static"

# External pressure load to astros that we will inherited from su2
load = {"loadType" : "PressureExternal"}
astros.input.Load = {"pressureAero": load}

# Set constraints
constraint = {"groupName"     : ["rootConstraint"],
              "dofConstraint" : 123456}
astros.input.Constraint = {"rootConstraint": constraint}

# Set materials
unobtainium  = {"youngModulus" : 2.2e13,
                "poissonRatio" : .5,
                "density"      : 7850}
madeupium    = {"materialType" : "isotropic",
                "youngModulus" : 1.2e14,
                "poissonRatio" : .5,
                "density"      : 7850}
astros.input.Material = {"Unobtainium": unobtainium,
                         "Madeupium"  : madeupium}

# Set properties
skinShell = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.05}
ribShell  = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.1}
sparShell = {"propertyType"        : "Shell",
             "material"            : "madeupium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.2}
astros.input.Property = {"leftWingSkin": skinShell,
                         "riteWingSkin": skinShell,
                         "wingRib"     :  ribShell,
                         "wingSpar1"   : sparShell,
                         "wingSpar2"   : sparShell}

#------------------------------------------------------------------------------#

# Create the data transfer connections
boundNames = ["upperWing", "lowerWing", "leftTip", "riteTip"]
for boundName in boundNames:
    # Create the bound
    bound = capsProblem.bound.create(boundName)

    # Create the vertex sets on the bound for fun3d and astros analysis
    fun3dVset    = bound.vertexSet.create(fun3d)
    astrosVset = bound.vertexSet.create(astros)

    # Create pressure data sets
    fun3d_Pressure    = fun3dVset.dataSet.create("Pressure")
    astros_Pressure = astrosVset.dataSet.create("Pressure")

    # Create displacement data sets
    fun3d_Displacement    = fun3dVset.dataSet.create("Displacement", init=[0,0,0])
    astros_Displacement = astrosVset.dataSet.create("Displacement")

    # Link the data sets
    astros_Pressure.link(fun3d_Pressure, "Conserve")
    fun3d_Displacement.link(astros_Displacement, "Interpolate")

    # Close the bound as complete (cannot create more vertex or data sets)
    bound.close()

#------------------------------------------------------------------------------#

# Declare ASTROS install directory
astrosInstallDir = os.environ['ESP_ROOT'] + os.sep + "bin" + os.sep

# Copy files needed to run astros
files = ["ASTRO.D01", "ASTRO.IDX"]
for file in files:
    try:
        shutil.copy2(astrosInstallDir + file, astros.analysisDir + os.sep + file)
    except:
        print ("Unable to copy \"", file, "\"")
        raise SystemError

# Aeroelastic iteration loop
for iter in range(numTransferIteration):

    ####### Fun3D ###########################
    print ("\n==> Running Fun3D pre-analysis...")
    fun3d.preAnalysis()

    #------------------------------
    print ("\n==> Running Fun3D......")

    cmdLineOpt = "--write_aero_loads_to_file --animation_freq -1"
    if iter != 0:
        cmdLineOpt = cmdLineOpt + " --read_surface_from_file"

    fun3d.system("mpirun -np 1 nodet_mpi " + cmdLineOpt + " > Info.out"); # Run fun3d via system call
    #------------------------------

    print ("\n==> Running Fun3D post-analysis...")
    fun3d.postAnalysis()
    #######################################


    ####### ASTROS ########################
    print ("\n==> Running ASTROS pre-analysis...")
    astros.preAnalysis()

    #------------------------------
    # Run mASTROS via system call
    print ("\n==> Running mASTROS......")
    astros.system("mastros.exe < " + astros.input.Proj_Name +  ".dat > " + astros.input.Proj_Name + ".out");
    #------------------------------

    print ("\n==> Running ASTROS post-analysis...")
    astros.postAnalysis()
    #######################################

